@@ -262,20 +262,20 @@ class PaddedBcdAdapter(BcdAdapter):
         return super()._encode(obj, context, path)
 
 class PlmnAdapter(BcdAdapter):
-    """convert a bytes(3) type to BCD string like 262-02 or 262-002."""
+    """convert a bytes(3) type to BCD string like 262-02 or 262-002 as specified in 3GPP TS 24.008 § 10.5.1.3."""
     def _decode(self, obj, context, path):
         bcd = super()._decode(obj, context, path)
         if bcd[3] == 'f':
             return '-'.join([bcd[:3], bcd[4:]])
         else:
-            return '-'.join([bcd[:3], bcd[3:]])
+            return '-'.join([bcd[:3], bcd[4:6]+bcd[3]])
 
     def _encode(self, obj, context, path):
         l = obj.split('-')
         if len(l[1]) == 2:
             bcd = l[0] + 'f' + l[1]
         else:
-            bcd = l[0] + l[1]
+            bcd = l[0] + l[1][2] + l[1][0:2]
         return super()._encode(bcd, context, path)
 
 class InvertAdapter(Adapter):
@@ -413,16 +413,20 @@ class StripTrailerAdapter(Adapter):
     Encoder removes all trailing bytes matching the default_value
     Decoder pads input data up to total_length with default_value
 
+    In case the encoding restricts the length of the result to specific values, the API user may set those restrictions
+    using the steps parameter. (e.g. encoded result must be either 1 or 3 byte long, steps would be set to [1,3])
+
     This is used in constellations like "FlagsEnum(StripTrailerAdapter(GreedyBytes, 3), ..."
     where you have a bit-mask that may have 1, 2 or 3 bytes, depending on whether or not any
     of the LSBs are actually set.
     """
-    def __init__(self, subcon, total_length:int, default_value=b'\x00', min_len=1):
+    def __init__(self, subcon, total_length:int, default_value=b'\x00', min_len=1, steps:typing.List[int]=[]):
         super().__init__(subcon)
         assert len(default_value) == 1
         self.total_length = total_length
         self.default_value = default_value
         self.min_len = min_len
+        self.steps = steps
 
     def _decode(self, obj, context, path):
         assert isinstance(obj, bytes)
@@ -435,10 +439,60 @@ class StripTrailerAdapter(Adapter):
         assert isinstance(obj, int)
         obj = obj.to_bytes(self.total_length, 'big')
         # remove trailing bytes if they are zero
+
+        obj_step_aligned = obj
         while len(obj) > self.min_len and obj[-1] == self.default_value[0]:
             obj = obj[:-1]
-        return obj
+            if len(obj) in self.steps:
+                obj_step_aligned = obj
 
+        if self.steps == []:
+            return obj
+        else:
+            return obj_step_aligned;
+
+class StripHeaderAdapter(Adapter):
+    """
+    Encoder removes all leading bytes matching the default_value
+    Decoder pads input data up to total_length with default_value
+
+    In case the encoding restricts the length of the result to specific values, the API user may set those restrictions
+    using the steps parameter. (e.g. encoded result must be either 1 or 3 byte long, steps would be set to [1,3])
+
+    This is used in constellations like "FlagsEnum(StripHeaderAdapter(GreedyBytes, 3), ..."
+    where you have a bit-mask that may have 1, 2 or 3 bytes, depending on whether or not any
+    of the MSBs are actually set.
+    """
+    def __init__(self, subcon, total_length:int, default_value=b'\x00', min_len=1, steps:typing.List[int]=[]):
+        super().__init__(subcon)
+        assert len(default_value) == 1
+        self.total_length = total_length
+        self.default_value = default_value
+        self.min_len = min_len
+        self.steps = steps
+
+    def _decode(self, obj, context, path):
+        assert isinstance(obj, bytes)
+        # pad with suppressed/missing bytes
+        if len(obj) < self.total_length:
+            obj = self.default_value * (self.total_length - len(obj)) + obj
+        return int.from_bytes(obj, 'big')
+
+    def _encode(self, obj, context, path):
+        assert isinstance(obj, int)
+        obj = obj.to_bytes(self.total_length, 'big')
+        # remove trailing bytes if they are zero
+
+        obj_step_aligned = obj
+        while len(obj) > self.min_len and obj[0] == self.default_value[0]:
+            obj = obj[1:]
+            if len(obj) in self.steps:
+                obj_step_aligned = obj
+
+        if self.steps == []:
+            return obj
+        else:
+            return obj_step_aligned;
 
 def filter_dict(d, exclude_prefix='_'):
     """filter the input dict to ensure no keys starting with 'exclude_prefix' remain."""
